package cz.muni.fi.lasaris.sbms.data;

import java.util.Properties;

import org.apache.log4j.Logger;

import cz.muni.fi.lasaris.sbms.data.connectors.ArchiveConnector;
import cz.muni.fi.lasaris.sbms.data.connectors.BASConnector;


public class ConnectorProvider {
	final static Logger logger = Logger.getLogger(ConnectorProvider.class);
	private static ArchiveConnector aConnector;
	private static BASConnector bConnector;
	
	public static void init(Properties props) {
		try {
			String bClassName = props.getProperty("data.connectors.bas");
			String aClassName = props.getProperty("data.connectors.archive");
			if(bClassName != null) {
				Class<?> c = Class.forName(bClassName);
				ConnectorProvider.bConnector = (BASConnector)c.newInstance();
				ConnectorProvider.bConnector.init(props);
				logger.debug(String.format("BAS connector %s initialized.", bClassName));
			} else {
				throw new IllegalArgumentException(String.format("Unable to initialize BAS data provider %s.", bClassName));
			}
			if(aClassName != null) {
				Class<?> c = Class.forName(aClassName);
				ConnectorProvider.aConnector = (ArchiveConnector)c.newInstance();
				ConnectorProvider.aConnector.init(props);
				logger.debug(String.format("Archive connector %s initialized.", aClassName));
			} else {
				throw new IllegalArgumentException(String.format("Unable to initialize Archive data provider %s."));
			}
		} catch (Exception ex) {
			logger.error("Unable to initialize data providers.");
			logger.error(ex);
		}
	}
	
	public static BASConnector getBASConnector() {
		if(ConnectorProvider.bConnector == null) {
			throw new IllegalStateException("connector not set");
		}
		return ConnectorProvider.bConnector;
	}
	
	public static ArchiveConnector getArchiveConnector() {
		if(ConnectorProvider.aConnector == null) {
			throw new IllegalStateException("connector not set");
		}
		return ConnectorProvider.aConnector;
	}
	
	public static void close() {
		ConnectorProvider.aConnector.close();
		ConnectorProvider.bConnector.close();
	}
}
