package cz.muni.fi.lasaris.sbms.data.connectors;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import cz.muni.fi.lasaris.sbms.data.structures.Address;
import cz.muni.fi.lasaris.sbms.data.structures.AddressGroup;
import cz.muni.fi.lasaris.sbms.data.structures.AggregationFunction;
import cz.muni.fi.lasaris.sbms.data.structures.SeriesSpecs;
import cz.muni.fi.lasaris.sbms.data.structures.containers.Aggregation;
import cz.muni.fi.lasaris.sbms.data.structures.containers.Series;
import cz.muni.fi.lasaris.sbms.data.structures.containers.Slice;
import cz.muni.fi.lasaris.sbms.data.structures.containers.Trend;
import cz.muni.fi.lasaris.sbms.data.structures.containers.TrendGroup;
import cz.muni.fi.lasaris.sbms.data.structures.values.AggregatedValue;
import cz.muni.fi.lasaris.sbms.data.structures.values.InterpolatedValue;

public interface ArchiveConnector {
		
		public void init(Properties props);
		
		public void close();
	
		// returns time series - either with interpolation or raw data
		public Trend getTrend(Address trend, SeriesSpecs s);
		
		// complete trend data for multiple trends - used for custom aggregation or direct inspection 
		public TrendGroup getTrends(List<Address> trends, SeriesSpecs s);
		
		// trend data provided in groups - used for custom aggregations
		public Map<AddressGroup, TrendGroup> getTrendsByGroup(List<AddressGroup> trends, SeriesSpecs s);
		
		// returns aggregated value for given trend
		public AggregatedValue getTrendAggregation(Address trend, SeriesSpecs seriesSpecs, AggregationFunction aggregation);
		
		// returns aggregated values for each of the trends
		public Aggregation<Address> getTrendAggregations(List<Address> trend, SeriesSpecs seriesSpecs, AggregationFunction aggregation);
		
		// aggregations over time and groups
		public Aggregation<AddressGroup> getGroupTrendAggregations(List<AddressGroup> trend, SeriesSpecs seriesSpecs, AggregationFunction aggregation);
		
		// methods that might perform some sort of interpolation - return one value for one moment in time
		public InterpolatedValue getTrendValue(Address trend, LocalDateTime timestamp);
		
		// returns slice - for each address one value at give time (interpolation will likely occur)
		public Slice getSlice(List<Address> trends, LocalDateTime timestamp);
		
		// returns "slices" - values of trends in given time (equivalent to repeated call to getTrendValues with different timestamps)
		public Series<Slice> getSlices(List<Address> trends, SeriesSpecs seriesSpecs);
		
		// creates aggregation over "slices" - performs aggregation on the slice, not on the individual trend
		public Series<AggregatedValue> getSliceAggregations(List<Address> trends, SeriesSpecs seriesSpecs, AggregationFunction aggregation);
		
		// aggregations over slice and group
		public Series<Aggregation<AddressGroup>> getGroupSliceAggregations(List<AddressGroup> trends, SeriesSpecs seriesSpecs, AggregationFunction aggregation);
		
}
