package cz.muni.fi.lasaris.sbms.data.structures.values;

import cz.muni.fi.lasaris.sbms.data.structures.AggregationFunction;
import cz.muni.fi.lasaris.sbms.data.structures.DataType;
import cz.muni.fi.lasaris.sbms.data.structures.Interpolation;

public abstract class Value {
	private Object value;
	protected AggregationFunction aggregation;
	protected Interpolation interpolation;
	private DataType  dataType;
	
	public Object getValue() {
		return value;
	}
	
	public int getValueAsInt() {
		return ((Integer)value).intValue();
	}
	
	public double getValueAsDouble() {
		return ((Double)value).doubleValue();
	}
	
	public boolean getValueAsBoolean() {
		return ((Boolean)value).booleanValue();
	}
	
	public String getValueAsString() {
		return value.toString();
	}
	
	public void setValue(Object value) {
		this.value = value;
	}
	
	public AggregationFunction getAggregation() {
		return aggregation;
	}
	
	public Interpolation getInterpolation() {
		return interpolation;
	}
	
	public DataType getDataType() {
		return dataType;
	}
	
	public void setDataType(DataType dataType) {
		this.dataType = dataType;
	}
	
	public boolean IsNull() {
		return value == null;
	}
	
	public Value(Object value, DataType dataType) {
		this.value = value;
		this.dataType = dataType;
	}
	
	protected Value(int value) {
		this.value = new Integer(value);
		this.dataType = DataType.INTEGER;
	}
	
	protected Value(double value) {
		this.value = new Double(value);
		this.dataType = DataType.REAL;
	}
	
	protected Value(boolean value) {
		this.value = new Boolean(value);
		this.dataType = DataType.BOOL;
		
	}
	
	protected Value(String value) {
		this.value = new String(value);
		this.dataType = DataType.TEXT;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((aggregation == null) ? 0 : aggregation.hashCode());
		result = prime * result + ((dataType == null) ? 0 : dataType.hashCode());
		result = prime * result + ((interpolation == null) ? 0 : interpolation.hashCode());
		result = prime * result + ((value == null) ? 0 : value.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Value other = (Value) obj;
		if (aggregation != other.aggregation)
			return false;
		if (dataType != other.dataType)
			return false;
		if (interpolation != other.interpolation)
			return false;
		if (value == null) {
			if (other.value != null)
				return false;
		} else if (!value.equals(other.value))
			return false;
		return true;
	}

	public String toString() {
		return value.toString();
	}

	
}
