package cz.muni.fi.lasaris.sbms.data.connectors.plugins;

import java.util.List;
import java.util.Properties;

import org.apache.log4j.Logger;

import com.serotonin.bacnet4j.LocalDevice;
import com.serotonin.bacnet4j.RemoteDevice;
import com.serotonin.bacnet4j.npdu.ip.IpNetworkBuilder;
import com.serotonin.bacnet4j.service.acknowledgement.ReadPropertyMultipleAck;
import com.serotonin.bacnet4j.service.confirmed.ConfirmedRequestService;
import com.serotonin.bacnet4j.service.confirmed.ReadPropertyMultipleRequest;
import com.serotonin.bacnet4j.service.unconfirmed.WhoIsRequest;
import com.serotonin.bacnet4j.transport.DefaultTransport;
import com.serotonin.bacnet4j.transport.Transport;
import com.serotonin.bacnet4j.type.constructed.Choice;
import com.serotonin.bacnet4j.type.constructed.ReadAccessSpecification;
import com.serotonin.bacnet4j.type.constructed.SequenceOf;
import com.serotonin.bacnet4j.type.enumerated.ObjectType;
import com.serotonin.bacnet4j.type.enumerated.PropertyIdentifier;
import com.serotonin.bacnet4j.type.primitive.ObjectIdentifier;

import cz.muni.fi.lasaris.sbms.data.connectors.BASConnector;
import cz.muni.fi.lasaris.sbms.data.structures.Address;
import cz.muni.fi.lasaris.sbms.data.structures.AddressGroup;
import cz.muni.fi.lasaris.sbms.data.structures.AggregationFunction;
import cz.muni.fi.lasaris.sbms.data.structures.containers.Aggregation;
import cz.muni.fi.lasaris.sbms.data.structures.containers.Snapshot;
import cz.muni.fi.lasaris.sbms.data.structures.values.AggregatedValue;
import cz.muni.fi.lasaris.sbms.data.structures.values.RawValue;

public class BACnetConnector implements BASConnector {
	
	final static Logger logger = Logger.getLogger(BACnetConnector.class);
	
	private static LocalDevice device;
	
	
	@Override
	public void init(Properties props) {
		if(device != null) {
			logger.debug("Init was already called (either on this or on another instance of BACnetConnector - ignoring the method call.");
			return;
		}
		
		IpNetworkBuilder nb = new IpNetworkBuilder();
		
		nb.localBindAddress(props.getProperty("data.connectors.bas.bacnet.ip.address"));
		nb.broadcastIp(props.getProperty("data.connectors.bas.bacnet.ip.broadcast"));
		nb.subnetMask(props.getProperty("data.connectors.bas.bacnet.ip.mask"));
		
		if(null != props.getProperty("data.connectors.bas.bacnet.tcp.port")) {
			nb.port(Integer.parseInt(props.getProperty("data.connectors.bas.bacnet.tcp.port")));
		}
		Transport t = new DefaultTransport(nb.build());
		BACnetConnector.device = new LocalDevice(Integer.parseInt(props.getProperty("data.connectors.bas.bacnet.device.id")), t);
		try {
			
			device.initialize();
			// TODO - testing only, replace with sequence of whois
			device.sendGlobalBroadcast(new WhoIsRequest());
			logger.debug("BACnet device initialized.");
		} catch (Exception e) {
			logger.error("BACnet device was unable to initialize.");
			logger.error(e);
		}
		
	}

	
	public void close() {
		if(device != null) {
			device.terminate();
		}
	}

	public BACnetConnector() {
		
		if(device == null) {
			//throw new IllegalStateException("BACnet device was not initilaized.");
			logger.debug("First instance of the BACnetConnector created - BACnet layer not initialized yet.");
		}
		
		logger.debug("Instance of BACnetConnector created.");
	}

	@Override
	public RawValue getDataPointValue(Address dataPoint) {
		try {
		BACnetAddress a = new BACnetAddress(dataPoint);
		RemoteDevice rd = device.getRemoteDevice(a.getDevice());
		SequenceOf<ReadAccessSpecification> specs = new SequenceOf<ReadAccessSpecification>();
		ReadAccessSpecification ras = new ReadAccessSpecification(
				new ObjectIdentifier(new ObjectType(a.getType()), a.getInstance()), 
				new PropertyIdentifier(a.getProperty()));
		specs.add(ras);
		ConfirmedRequestService crs = new ReadPropertyMultipleRequest(specs);
		logger.debug("Sending ReadPropMulti request...");
		ReadPropertyMultipleAck result = device.send(rd, crs).get();
		logger.debug("ReadPropMulti response received");
		Choice c = result.getListOfReadAccessResults().get(0).getListOfResults().get(0).getReadResult();
		switch(dataPoint.getDataType()) {
		case BOOL:
			com.serotonin.bacnet4j.type.primitive.Boolean b = c.getDatum();
			return new RawValue(b.booleanValue());
		case INTEGER:
			com.serotonin.bacnet4j.type.primitive.SignedInteger i = c.getDatum();
			return new RawValue(i.intValue());
		case REAL:
			com.serotonin.bacnet4j.type.primitive.Real r = c.getDatum();
			return new RawValue((double)r.floatValue());
		case TEXT:
		default:
			com.serotonin.bacnet4j.type.primitive.CharacterString t = c.getDatum();
			return new RawValue(t.getValue());
		}
		} catch(Exception e) {
			logger.debug("Unable to read object " + dataPoint);
			logger.debug(e);
			return null;
		}
	}

	@Override
	public RawValue getDataPointValue(Address dataPoint, boolean allowCache) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public Snapshot getDataPointValues(List<Address> dataPoints) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public Snapshot getDataPointValues(List<Address> dataPoints, boolean allowCache) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public AggregatedValue getDataPointsAggregation(List<Address> dataPoints, AggregationFunction aggregation) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public AggregatedValue getDataPointsAggregation(List<Address> dataPoints, AggregationFunction aggregation,
			boolean allowCache) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public Aggregation<AddressGroup> getDataPointGroupAggregations(List<AddressGroup> dataPointGroups,
			AggregationFunction aggregation) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public Aggregation<AddressGroup> getDataPointGroupAggregations(List<AddressGroup> dataPointGroups,
			AggregationFunction aggregation, boolean allowCache) {
		// TODO Auto-generated method stub
		return null;
	}

}
